<?php

namespace Drupal\Tests\view_mode_switch\Kernel;

use Drupal\Core\Entity\Entity\EntityViewMode;
use Drupal\entity_test\Entity\EntityTest;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\system\Form\RssFeedsForm;

/**
 * Tests the view mode helper for the view mode switch field type.
 *
 * @group view_mode_switch
 * @coversDefaultClass \Drupal\view_mode_switch\ViewModeHelper
 */
class ViewModeHelperTest extends ViewModeSwitchTestBase {

  /**
   * The entity form builder.
   *
   * @var \Drupal\Core\Entity\EntityFormBuilder
   */
  protected $entityFormBuilder;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The view mode helper.
   *
   * @var \Drupal\view_mode_switch\ViewModeHelperInterface
   */
  protected $viewModeHelper;

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'field_ui',
    'view_mode_switch_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->entityFormBuilder = $this->container->get('entity.form_builder');
    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->formBuilder = $this->container->get('form_builder');
    $this->viewModeHelper = $this->container->get('view_mode_switch.view_mode_helper');
  }

  /**
   * Tests field storage updates appear in view mode delete form (if necessary).
   *
   * @covers ::confirmFormAlter
   */
  public function testConfirmFormAlter() {
    /* @var \Drupal\field\FieldStorageConfigInterface $field_storage_foo */
    $field_storage_foo = $this->fieldFoo->getFieldStorageDefinition();
    $entity_type_id = $field_storage_foo->getEntityTypeId();
    $entity_type = $this->entityTypeManager->getDefinition($entity_type_id);
    $entity_type_label = $entity_type->getLabel();

    // Test delete confirm form of view mode that is used as origin view mode.
    $form = $this->entityFormBuilder->getForm($this->viewModeFoo, 'delete');
    $this->assertIdentical($form['entity_updates']['field_storage_config']['#items'], [$field_storage_foo->id() => $field_storage_foo->id()]);
    $this->assertEquals($entity_type_label, $form['entity_updates']['field_storage_config']['#title']);

    // Test delete confirm form of view mode that is not used as origin view
    // mode.
    $form = $this->entityFormBuilder->getForm($this->viewModeFoo1, 'delete');
    $this->assertTrue(!isset($form['entity_updates']['field_storage_config']['#items'][$field_storage_foo->id()]));
  }

  /**
   * Tests exception thrown by view mode delete form alter in wrong context.
   *
   * In this test the form alter is tried to be applied on an 'Entity test'
   * entity delete confirm form.
   *
   * @see \view_mode_switch_test_form_entity_test_entity_test_delete_form_form_alter()
   *
   * @covers ::confirmFormAlter
   */
  public function testConfirmFormAlterExceptionsForWrongEntityType() {
    // Test exception when form alter is applied to 'delete' operation entity
    // form for wrong entity type.
    $this->expectException(\LogicException::class);
    $this->expectExceptionMessage('Required form alters for potential field storage configuration updates may only be applied to view mode entity forms.');
    $this->entityFormBuilder->getForm(EntityTest::create(['view_mode_switch_form_alter_test' => TRUE]), 'delete');
  }

  /**
   * Tests exception thrown by view mode delete form alter in wrong context.
   *
   * In this test the form alter is tried to be applied on an 'Entity view mode'
   * entity edit form.
   *
   * @see \view_mode_switch_test_form_entity_view_mode_edit_form_alter()
   *
   * @covers ::confirmFormAlter
   */
  public function testConfirmFormAlterExceptionsForWrongOperation() {
    // Test exception when form alter is applied to wrong operation entity form.
    $this->expectException(\LogicException::class);
    $this->expectExceptionMessage('Required form alters for potential field storage configuration updates may only be applied to view mode entity delete forms.');
    $this->entityFormBuilder->getForm(EntityViewMode::create([
      'targetEntityType' => 'entity_test',
      'id' => 'entity_test.test_wrong_operation',
      'view_mode_switch_form_alter_test' => TRUE,
    ]), 'edit');
  }

  /**
   * Tests exception thrown by view mode delete form alter in wrong context.
   *
   * In this test the form alter is tried to be applied on a non-entity form.
   *
   * @see \view_mode_switch_test_form_system_rss_feeds_settings_alter()
   *
   * @covers ::confirmFormAlter
   */
  public function testConfirmFormAlterExceptionsForNonEntityForm() {
    // Test exception when form alter is applied to wrong form.
    $this->expectException(\LogicException::class);
    $this->expectExceptionMessage('Required form alters for for potential field storage configuration updates may only be applied to entity forms.');
    $this->formBuilder->getForm(RssFeedsForm::class, ['view_mode_switch_form_alter_test' => TRUE]);
  }

  /**
   * Tests getting the view mode name based on its ID.
   *
   * @covers ::getName
   */
  public function testGetName() {
    $expected_name = 'test_get_name';

    /* @var \Drupal\Core\Entity\EntityViewModeInterface $entity */
    $entity = EntityViewMode::create([
      'id' => 'entity_test.' . $expected_name,
    ]);

    $this->assertEquals($expected_name, $this->viewModeHelper->getName($entity));
  }

  /**
   * Tests origin view mode deletion changing view mode switch field storages.
   *
   * @covers ::preDelete
   */
  public function testPreDelete() {
    $view_mode_bar = $this->getViewModeNameFromId($this->viewModeBar->id());
    $view_mode_baz = $this->getViewModeNameFromId($this->viewModeBaz->id());

    // Test initial origin view modes of 'bar_baz' field.
    /* @var \Drupal\field\FieldStorageConfigInterface $field_storage_bar_baz */
    $field_storage_bar_baz = $this->fieldBarBaz->getFieldStorageDefinition();
    $origin_view_modes = $field_storage_bar_baz->getSetting('origin_view_modes');
    $this->assertNotFalse(array_search($view_mode_bar, $origin_view_modes));
    $this->assertNotFalse(array_search($view_mode_baz, $origin_view_modes));

    // Test origin view mode removal behavior on 'bar_baz' field when 'bar'
    // origin view mode is deleted (field storage configuration should be
    // updated by removing configured view mode as well).
    $this->viewModeBar->delete();
    $field_storage_bar_baz = FieldStorageConfig::load($field_storage_bar_baz->id());
    $origin_view_modes = $field_storage_bar_baz->getSetting('origin_view_modes');
    $this->assertFalse(array_search($view_mode_bar, $origin_view_modes));
    $this->assertNotFalse(array_search($view_mode_baz, $origin_view_modes));

    // Test origin view mode removal behavior on 'bar_baz' field when 'baz'
    // origin view mode is deleted (field storage configuration should be
    // updated by removing configured view mode as well).
    $this->viewModeBaz->delete();
    $field_storage_bar_baz = FieldStorageConfig::load($field_storage_bar_baz->id());
    $origin_view_modes = $field_storage_bar_baz->getSetting('origin_view_modes');
    $this->assertFalse(array_search($view_mode_bar, $origin_view_modes));
    $this->assertFalse(array_search($view_mode_baz, $origin_view_modes));
    $this->assertCount(0, $origin_view_modes);
  }

}
